import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import "../components"
import "./view"

import Artix.Handlers
import Artix.Core
import Artix.Gui

/**
 * @brief Диалог ввода текста
 * @param fullScreen диалог является полноэкранным
 */
BasicView {
    id: root

    padding: 32
    implicitWidth: screenWidth
    implicitHeight: screenHeight

    property bool moneyItem: params.moneyItem   ///< необходимо ввести денежное значение
    property bool returnCancelButtonText: params.returnCancelButtonText ///< возвращать текст кнопки при нажатии отмены
    property bool forcedVerifiedInput: params.forcedVerifiedInput ///< выпонять проверку введенного текста по маске
    property var cancelButtonText: params.cancelBtnText ///< перевод для кнопки отмены (не используется, нужна для совместимоси)

    readonly property bool fullScreen: true

    property BasicTextField eventReceiver: inputField
    property DialogEventFilter eventFilter: DialogEventFilter {
        onKeyboardData: (data, rawData, source) => {
                            inputField.clear()
                            handler.input(data, rawData, source)
                        }
        onOk: accept()
        onCancel: reject()
    }

    header: Item {
        TopPanel { anchors.fill: parent; anchors.leftMargin: 24 }
        height: 72
    }
    contentItem: ColumnLayout {
        spacing: 0
        BasicLabel {
            Layout.preferredHeight: 24
            Style.class: "inputDialogMessage"
            Tr { tr: params.message }
        }
        Item { Layout.preferredHeight: 24 }
        BasicTextField {
            id: inputField
            Layout.fillWidth: true
            inputMethodHints: Qt.ImhNoPredictiveText
            Layout.preferredHeight: 64
            Style.class: (!forcedVerifiedInput || acceptableInput) ? "inputDialogTextFiled"
                                                                   : "inputDialogTextFiledInvalid"
            padding: 12
            echoMode: params.password ? TextInput.Password : TextInput.Normal
            text: params.defaultValue
            inputMask: params.inputPattern
            validator: params.inputMask ? validator : null
            Image {
                anchors.right: parent.right
                anchors.rightMargin: 12
                anchors.verticalCenter: parent.verticalCenter
                width: 30
                height: 30
                source: Style.parameter("icon.textFieldClear")
                MouseArea { anchors.fill: parent; onClicked: inputField.clear() }
            }
        }
        Item { Layout.fillHeight: true }
        InlineKeyboard {
            property bool digital: params.onlyNumber
            Layout.preferredWidth: digital ? 256 : -1
            Layout.fillWidth: !digital
            Layout.preferredHeight: 372
            Layout.alignment: Qt.AlignHCenter
            layoutType: digital ? "input_digits" : "input"
            defaultLocale: "ru_RU"
            enterEnabled: !forcedVerifiedInput || inputField.acceptableInput
            onEnter: root.accept()
            onCancel: root.reject()

            showCancel: !params.hideCancel
        }
    }
    footer: BottomPanel{}

    RegularExpressionValidator {
        id: validator
        regularExpression: new RegExp(params.inputMask)
    }
    DialogHandler {
        id: handler
        allowDevices: params.allowDevices
    }

    function accept() {
        handler.sendAnswer((moneyItem ? inputField.text.replace(" ","") : inputField.text), InputSource.Keyboard);
    }

    function reject() {
        if (returnCancelButtonText)
            handler.sendAnswer(cancelButtonText.ui(), InputSource.Undefined)
        else
            handler.sendAnswer("", InputSource.Undefined)
    }
}
