import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import "../components"
import "view"
import "view/delegate"

import Artix.Core
import Artix.Gui
import Artix.Models
import Artix.Handlers

/**
 * @brief Диалог выбора цены
 */
ChoiceList {
    id: root
    cancelVisible: params.showCancel
    dataModel: IndexPriceModel { tmc: params.tmc }
    delegate: ChoiceListDelegate {
        contentItem: Item {
            BasicLabel {
                id: delegateCode
                Style.class: "priceChoiceDelegateCode"
                anchors { top: delegateName.top; left: parent.left }
                width: 20
                text: code
            }
            BasicLabel {
                id: delegateName
                anchors {
                    left: delegateCode.right; right: delegatePrice.left
                    leftMargin: 8; rightMargin: 48
                }
                Style.class: "priceChoiceDelegateName"
                anchors.verticalCenter: parent.verticalCenter
                text: name
            }
            BasicLabel {
                id: delegatePrice
                Style.class: "priceChoiceDelegatePrice"
                anchors { verticalCenter: parent.verticalCenter; right: parent.right }
                width: 172
                horizontalAlignment: Qt.AlignRight
                text: price
            }
        }
    }
    titleContent: ColumnLayout {
        spacing: 4
        BasicLabel {
            Style.class: "priceChoiceTitle"
            Layout.fillWidth: true
            Tr { tr: params.message }
        }
        BasicLabel {
            Layout.fillWidth: true
            Style.class: "priceChoiceTmcName"
            text: Utils.variantToObjectPointer(params.tmc).name
        }
    }

    DialogHandler { id: handler }

    onAccept: handler.sendAnswer(dataModel.data(selectionModel.currentIndex, IndexPriceModel.CodeRole))
    onReject: handler.sendAnswer(-1)
}
