import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes
import Qt.labs.qmlmodels

import "../../components"
import "./delegate"

import Artix.Handlers
import Artix.Models
import Artix.Core
import Artix.Gui

/**
 * @brief Список позиций документа
 */
ListView {
    id: root

    property bool scrollToEndOnItemAdded: false
    property bool interactive: true
    property alias withModifiers: dm.withModifiers
    signal storno(var position)
    signal changeQuantity(var position, real value)
    signal consultantDelete(var position)

    function scrollToEnd() {
        let index = count - 1
        positionViewAtEnd()
        currentIndex = index
    }

    onCountChanged: {
        if (root.scrollToEndOnItemAdded)
            root.scrollToEnd()
    }

    clip: true
    model: DocumentModel { id: dm }
    // !!!!!!!!!! Делегат товара !!!!!!!!!!
    component GoodsItemDelegate: PositionDelegate {
        id: gi
        property bool hasImage: image !== undefined

        interactive: root.interactive
        width: root.width
        height: (selected ? contentItem.implicitHeight : giCommonContent.implicitHeight) + topPadding + bottomPadding
                + (object.hasConsultant ? 24 : 0)
        contentItem: ColumnLayout {
            spacing: 0
            ColumnLayout {
                visible: object.hasConsultant
                Layout.bottomMargin: 12
                RowLayout {
                    Image {
                        Layout.preferredWidth: 12
                        Layout.preferredHeight: 14
                        source: Style.parameter("icon.position.consultant")
                    }
                    BasicLabel {
                        Layout.fillWidth: true
                        color: Style.parameter("color.info")
                        text: consultantName ? consultantName : ""
                        elide: Text.ElideRight
                        wrapMode: Text.WordWrap
                        maximumLineCount: 1
                    }
                }
            }
            RowLayout {
                id: giCommonContent
                spacing: 12
                Layout.alignment: Qt.AlignTop
                Image {
                    Layout.preferredWidth: 120
                    Layout.preferredHeight: 120
                    fillMode: Image.PreserveAspectFit
                    visible: gi.hasImage
                    source: gi.hasImage ? image : ""
                }
                ColumnLayout {
                    spacing: 16
                    RowLayout {
                        spacing: 0
                        BasicLabel {
                            Layout.fillWidth: true
                            Layout.alignment: Qt.AlignTop
                            Layout.preferredHeight: gi.hasImage ? 84 : 40
                            Style.class: "positionDelegateName"
                            text: object.name
                            elide: Text.ElideRight
                        }
                        ColumnLayout {
                            Layout.alignment: Qt.AlignTop
                            BasicLabel {
                                Layout.preferredWidth: 138
                                Layout.preferredHeight: 21
                                Style.class: "positionDelegateSum"
                                text: Utils.formatMoneyItem(object.sumb)
                                horizontalAlignment: Text.AlignRight
                            }
                            ListView {
                                Layout.preferredHeight: 24
                                Layout.preferredWidth: 138
                                layoutDirection: ListView.RightToLeft
                                orientation: ListView.Horizontal
                                spacing: 4
                                model: tags
                                delegate: Image {
                                    width: 24
                                    height: 24
                                    fillMode: Image.PreserveAspectFit
                                    source: Style.parameter(url, url)
                                }
                            }
                        }
                    }
                    RowLayout {
                        spacing: 0
                        BasicLabel {
                            Style.class: "positionDelegateQuant"
                            Layout.preferredHeight: 20
                            text: {
                                let quant = object.unitFlag ? Utils.weightStr(object.bquant) : object.bquant.toFixed(0)
                                return quant + " x " + Utils.formatMoneyItem(object.price)
                            }
                        }
                        BasicLabel {
                            Style.class: "positionDelegateDisc"
                            Layout.preferredHeight: 20
                            visible: !Utils.moneyIsZero(object.disc_abs)
                            text: " - " + Utils.formatMoneyItem(object.disc_abs)
                        }
                        Item { Layout.preferredWidth: 12 }
                        BasicLabel {
                            Style.class: "positionDelegateDiscName"
                            elide: Text.ElideRight
                            Layout.fillWidth: true
                            Layout.preferredHeight: 20
                            padding: 8
                            topPadding: 1
                            bottomPadding: 3
                            visible: discountName !== undefined
                            text: discountName ? discountName : ""
                        }
                    }
                }
            }
            ColumnLayout {
                id: giSelectedContent
                opacity: selected ? 1 : 0
                spacing: 0
                Item { Layout.preferredHeight: 16; visible: campaingName !== undefined }
                BasicLabel {
                    Style.class: "positionDelegateCampaignTitle"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 16
                    visible: campaingName !== undefined
                    Tr { code: "positionDiscount"; defaultMessage: "Акция" }
                }
                Item { Layout.preferredHeight: 4; visible: campaingName !== undefined }
                BasicLabel {
                    Style.class: "positionDelegateCampaignName"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 20
                    maximumLineCount: 2
                    visible: campaingName !== undefined
                    text: campaingName ? campaingName : ""
                }
                Item { Layout.preferredHeight: 24 }
                BasicButton {
                    visible: object.hasConsultant
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    Style.class: "consultantDelete"
                    textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                    spacing: 16
                    Tr { code: "positionConsultantDelete"; defaultMessage: "Удалить консультанта" }
                    onClicked: consultantDelete(position)
                }
                Item {
                    visible: object.hasConsultant
                    Layout.preferredHeight: 24
                }
                RowLayout {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 56
                    spacing: 16
                    BasicButton {
                        Layout.preferredWidth: 72
                        Layout.preferredHeight: 56
                        Style.class: "positionQuant"
                        icon: Style.parameter("icon.minus")
                        enabled: Utils.weightGreater(object.bquant, 1.0) && object.unitFlag !== true
                        onClicked: changeQuantity(position, -1.0)
                    }
                    BasicButton {
                        Layout.fillWidth: true
                        Layout.preferredHeight: 56
                        Style.class: "positionStorno"
                        textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                        Tr { code: "positionStorno"; defaultMessage: "Удалить" }
                        onClicked: storno(position)
                    }
                    BasicButton {
                        Layout.preferredWidth: 72
                        Layout.preferredHeight: 56
                        Style.class: "positionQuant"
                        icon: Style.parameter("icon.plus")
                        enabled: object.unitFlag !== true
                        onClicked: changeQuantity(position, 1.0)
                    }
                }
                Behavior on opacity { OpacityAnimator { duration: 500 } }
            }
        }
    }
    // !!!!!!!!!! Делегат карты !!!!!!!!!!
    component CardDelegate: PositionDelegate {
        id: card

        interactive: root.interactive
        width: root.width
        height: (selected ? contentItem.implicitHeight : cardCommonContent.implicitHeight) + topPadding + bottomPadding
        contentItem: ColumnLayout {
            spacing: 24
            RowLayout {
                id: cardCommonContent
                spacing: 0
                BasicLabel {
                    Layout.preferredHeight: 20
                    Style.class: "cardDelegateName"
                    text: object.card["cardGroup"]["name"] + " № " + object.card["number"]
                    verticalAlignment: Text.AlignVCenter
                    Layout.fillWidth: true
                    elide: Text.ElideRight
                }
                Item { Layout.preferredWidth: 12 }
                BasicLabel {
                    Layout.preferredHeight: 20
                    Style.class: "cardDelegateBalance"
                    leftPadding: 8
                    rightPadding: 8
                    text: Utils.formatMoneyItem(object.bonusBalance)
                    verticalAlignment: Text.AlignVCenter
                    opacity: selected ? 0 : 1
                    visible: !selected
                    Behavior on opacity { OpacityAnimator { duration: 500 } }
                }
                Item { Layout.preferredWidth: 12 }
                Image {
                    Layout.preferredHeight: 24
                    source: Style.parameter("icon.position.card")
                }
            }
            ColumnLayout {
                opacity: selected ? 1 : 0
                spacing: 24
                RowLayout {
                    spacing: 24
                    ColumnLayout {
                        Layout.alignment: Qt.AlignTop
                        BasicLabel {
                            Layout.preferredWidth: 160
                            Style.class: "cardDelegateBalanceTitle"
                            Tr { code: "positionCardBalance"; defaultMessage: "Баланс" }
                        }
                        BasicLabel {
                            Layout.preferredWidth: 160
                            Style.class: "cardDelegateBalanceValue"
                            text: Utils.formatMoneyItem(object.bonusBalance)
                        }
                    }
                    Line {
                        Layout.fillHeight: true
                        strokeStyle: ShapePath.DashLine
                        dashPattern: [3, 4]
                        color: Style.parameter("color.gray")
                        direction: LineDirection.Vertical
                    }
                    ColumnLayout {
                        Layout.alignment: Qt.AlignTop
                        BasicLabel {
                            Layout.preferredWidth: 190
                            Style.class: "cardDelegateClientTitle"
                            Tr { code: "positionClientName"; defaultMessage: "Клиент" }
                        }
                        BasicLabel {
                            Layout.preferredWidth: 190
                            Style.class: "cardDelegateClientValue"
                            text: object.card["client"] && object.card["client"]["name"].length > 0
                                  ? object.card["client"]["name"] : " - "
                        }
                        BasicLabel {
                            Layout.preferredWidth: 190
                            Style.class: "cardDelegatePhoneTitle"
                            Tr { code: "positionClientPhone"; defaultMessage: "Телефон" }
                        }
                        BasicLabel {
                            Layout.preferredWidth: 190
                            Style.class: "cardDelegatePhoneValue"
                            text: object.card["client"] && object.card["client"]["phoneNumber"].length > 0
                                  ? object.card["client"]["phoneNumber"] : " - "
                        }
                    }
                }
                BasicButton {
                    Layout.preferredWidth: 232
                    Layout.preferredHeight: 56
                    Layout.alignment: Qt.AlignHCenter
                    Style.class: "positionStorno"
                    textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                    Tr { code: "positionStorno"; defaultMessage: "Удалить" }
                    onClicked: storno(position)
                }
                Behavior on opacity { OpacityAnimator { duration: 500 } }
            }
        }
    }
    // !!!!!!!!!! Делегат купона !!!!!!!!!!
    component CouponDelegate: PositionDelegate {
        id: coupon

        property var dateBeg: object.dateBeg.toLocaleDateString(Qt.locale("en_US"), "dd.MM.yy")
        property var dateEnd: object.dateEnd.toLocaleDateString(Qt.locale("en_US"), "dd.MM.yy")
        interactive: root.interactive
        width: root.width
        height: (selected ? contentItem.implicitHeight : couponCommonContent.implicitHeight) + topPadding + bottomPadding
        contentItem: ColumnLayout {
            RowLayout {
                id: couponCommonContent
                BasicLabel {
                    Layout.fillWidth: true
                    Style.class: "couponDelegateName"
                    Tr { code: "positionCoupon"; defaultMessage: "Купон %1"; args: [ object.number ] }
                    elide: Text.ElideRight
                }
                Item { Layout.preferredWidth: 12 }
                Image {
                    Layout.preferredHeight: 24
                    source: Style.parameter("icon.position.coupon")
                }
            }
            ColumnLayout {
                opacity: selected ? 1 : 0
                spacing: 24
                RowLayout {
                    spacing: 24
                    ColumnLayout {
                        Layout.alignment: Qt.AlignTop
                        BasicLabel {
                            Layout.preferredWidth: 160
                            Style.class: "couponDelegateDateTitle"
                            Tr { code: "positionCouponValidityDate"; defaultMessage: "Действует" }
                        }
                        BasicLabel {
                            Layout.preferredWidth: 160
                            Style.class: "couponDelegateDateValue"
                            property string validityDate
                            Tr on validityDate {
                                code: "positionCouponValidityDateValue";
                                defaultMessage: "с %1 - по %2"
                                args: [ dateBeg, dateEnd ]
                            }
                            text: dateBeg.length > 0  || dateEnd.length > 0 ? validityDate : "-"
                        }
                    }
                    Line {
                        Layout.fillHeight: true
                        strokeStyle: ShapePath.DashLine
                        dashPattern: [3, 4]
                        color: Style.parameter("color.gray")
                        direction: LineDirection.Vertical
                    }
                    ColumnLayout {
                        Layout.alignment: Qt.AlignTop
                        BasicLabel {
                            Layout.preferredWidth: 190
                            Style.class: "couponDelegateCampaignTitle"
                            Tr { code: "positionCouponCampaign"; defaultMessage: "Условие" }
                        }
                        BasicLabel {
                            Layout.preferredWidth: 190
                            Style.class: "couponDelegateCampaignName"
                            text: object.campaignName.length > 0 ? object.campaignName : " - "
                        }
                    }
                }
                BasicButton {
                    Layout.preferredWidth: 232
                    Layout.preferredHeight: 56
                    Layout.alignment: Qt.AlignHCenter
                    Style.class: "positionStorno"
                    textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                    Tr { code: "positionStorno"; defaultMessage: "Удалить" }
                    onClicked: storno(position)
                }
                Behavior on opacity { OpacityAnimator { duration: 500 } }
            }
        }
    }
    // !!!!!!!!!! Делегат подарка !!!!!!!!!!
    component GiftDelegate: PositionDelegate {
        id: gift

        interactive: root.interactive
        width: root.width
        height: (selected ? contentItem.implicitHeight : giftCommonContent.implicitHeight) + topPadding + bottomPadding
        contentItem: ColumnLayout {
            RowLayout {
                id: giftCommonContent
                BasicLabel {
                    Layout.fillWidth: true
                    Style.class: "giftDelegateName"
                    text: object.name
                    elide: Label.ElideRight
                }
                Item { Layout.preferredWidth: 12 }
                BasicLabel {
                    Style.class: "giftDelegateQuant"
                    text: object.quantity.toFixed(0)
                }
                Item { Layout.fillWidth: true }
                Image {
                    Layout.preferredHeight: 24
                    source: Style.parameter("icon.position.gift")
                }
            }
        }
    }

    component ModifiersDelegate: PositionDelegate {
        id: modifiers

        interactive: root.interactive
        width: root.width
        height: (selected ? contentItem.implicitHeight : modifiersCommonContent.implicitHeight) + topPadding + bottomPadding
        contentItem: ColumnLayout {
            spacing: 40
            RowLayout {
                id: modifiersCommonContent
                spacing: 12
                Layout.preferredWidth: parent.width

                Repeater {
                    id: modifiersRepeater
                    model: ModifiersModel { modifiers: object }

                    delegate: RowLayout {
                        Layout.preferredWidth: parent.width / modifiersRepeater.count
                        Layout.maximumWidth: Layout.preferredWidth
                        Layout.preferredHeight: implicitHeight > parent.height ? implicitHeight : parent.height
                        spacing: modifiersCommonContent.spacing
                        property bool isLast: index === modifiersRepeater.count - 1
                        property bool showIcon: true
                        onWidthChanged: showIcon = modifiersRepeater.count < 3 && (width >= (implicitWidth + (showIcon ? 0 : 24 + spacing)))

                        ColumnLayout {
                            Layout.preferredHeight: implicitHeight > parent.height ? implicitHeight : parent.height
                            spacing: 16
                            BasicLabel {
                                Layout.alignment: Qt.AlignTop
                                Style.class: "modifierText"
                                Tr { tr: name }
                            }
                            BasicLabel {
                                visible: type !== ModifierType.PackingMode
                                Style.class: "modifierValue"
                                text: value
                            }
                        }

                        Image {
                            visible: showIcon
                            Layout.alignment: Qt.AlignTop | Qt.AlignRight
                            Layout.preferredWidth: 24
                            Layout.preferredHeight: 24
                            source: Style.parameter(icon)
                        }

                        Line {
                            visible: !isLast
                            direction: LineDirection.Vertical
                            strokeWidth: 1
                            strokeStyle: ShapePath.DashLine
                            dashPattern: [2, 2]
                            color: Style.parameter("color.gray")

                            Layout.fillHeight: true
                            Layout.alignment: Qt.AlignRight
                        }
                    }
                }
            }
            BasicButton {
                Layout.fillWidth: true
                Layout.preferredHeight: 56
                Layout.leftMargin: 88
                Layout.rightMargin: 88
                opacity: selected ? 1 : 0
                Style.class: "positionStorno"
                textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                Tr { code: "positionStorno"; defaultMessage: "Удалить" }
                onClicked: handler.emitAction("COMMAND_ESC", {})
                Behavior on opacity { OpacityAnimator { duration: 500 } }
            }
        }
    }
    // Выбор делегата в зависимости от типа позиции
    delegate: DelegateChooser {
        role: "type"
        DelegateChoice {
            roleValue: DocumentPositionType.Goods
            delegate: GoodsItemDelegate {}
        }
        DelegateChoice {
            roleValue: DocumentPositionType.Card
            delegate: CardDelegate {}
        }
        DelegateChoice {
            roleValue: DocumentPositionType.Coupon
            delegate: CouponDelegate {}
        }
        DelegateChoice {
            roleValue: DocumentPositionType.Gift
            delegate: GiftDelegate {}
        }
        DelegateChoice {
            roleValue: DocumentPositionType.Modifiers
            delegate: ModifiersDelegate {}
        }
    }
    // Блок "Итого"
    footerPositioning: ListView.PullBackFooter
    footer: GroupBox {
        Style.class: "positionsFooter"
        width: parent.width
        height: 120
        padding: 24
        visible: model.positionsCount > 0
        contentItem: ColumnLayout {
            spacing: 24
            RowLayout {
                BasicLabel {
                    Layout.fillWidth: true
                    Style.class: "positionsFooterDiscount"
                    Tr { code: "positionsDiscout"; defaultMessage: "Сумма скидок" }
                }
                BasicLabel {
                    Layout.fillWidth: true
                    Style.class: "positionsFooterDiscount"
                    text: Utils.formatMoneyItem(Session.document.discWithoutBonus)
                    horizontalAlignment: Text.AlignRight
                }
            }
            RowLayout {
                BasicLabel {
                    Layout.fillWidth: true
                    Style.class: "positionsFooterSum"
                    Tr { code: "positionsSum"; defaultMessage: "Сумма чека" }
                }
                BasicLabel {
                    Layout.fillWidth: true
                    Style.class: "positionsFooterSum"
                    text: Utils.formatMoneyItem(Session.document.sumb)
                    horizontalAlignment: Text.AlignRight
                }
            }
        }
    }
    ScrollBar.vertical: ScrollBar {
        id: scrollBar
        Timer {
            id: scrollTimer
            interval: 2500
            onTriggered: parent.active = false
        }
        onActiveChanged: scrollTimer.start()
    }
}
